
# q-expansion of newform 5292.2.a.s, downloaded from the LMFDB on 26 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 5292
weight = 2
poly_data = [-7, 0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0, 0], [0, 0], [0, 1], [0, 0], [0, 1], [2, 0], [0, 0], [-1, 0], [0, 3], [0, -2], [7, 0], [-3, 0], [0, 3], [-2, 0], [0, -2], [0, 4], [0, -2], [-8, 0], [-2, 0], [0, 1], [10, 0], [10, 0], [0, -6], [0, -1], [16, 0], [0, 4], [-1, 0], [0, 2], [-11, 0], [0, -4], [2, 0], [0, 6], [0, -2], [12, 0], [0, -8], [16, 0], [14, 0], [-18, 0], [0, 8], [0, -7], [0, -6], [16, 0], [0, -3], [2, 0], [0, 0], [25, 0], [8, 0], [7, 0], [0, -4], [-28, 0], [0, -8], [0, 6], [10, 0], [0, 4], [0, 5], [0, -5], [0, -5], [8, 0], [25, 0], [0, -6], [28, 0], [0, 0], [7, 0], [0, 2], [-6, 0], [0, -6], [32, 0], [-13, 0], [0, 3], [12, 0], [0, 9], [0, 2], [25, 0], [-11, 0], [-6, 0], [0, -12], [0, 12], [-28, 0], [0, 6], [-18, 0], [0, -8], [-15, 0], [0, 5], [14, 0], [0, 0], [0, 3], [0, -12], [41, 0], [0, 9], [20, 0], [0, -10], [0, -12], [-26, 0], [0, -9], [6, 0], [0, 2], [0, -4], [0, -13], [29, 0], [-11, 0], [-8, 0], [0, 12], [0, 14], [0, 12], [12, 0], [-32, 0], [0, 4], [0, 3], [0, 9], [14, 0], [36, 0], [-37, 0], [0, -10], [-17, 0], [6, 0], [0, -4], [-21, 0], [0, 6], [0, -2], [0, -7], [-10, 0], [-34, 0], [0, 15], [0, 11], [28, 0], [0, 14], [25, 0], [0, 18], [-28, 0], [-14, 0], [4, 0], [0, -15], [18, 0], [22, 0], [0, -20], [26, 0], [0, 3], [4, 0], [0, -9], [0, -18], [-49, 0], [0, -14], [-44, 0], [0, -3], [-38, 0], [0, -6], [-40, 0], [0, 3], [15, 0], [0, -18], [-18, 0], [0, -11], [-20, 0], [0, -6], [16, 0], [0, -10], [-36, 0], [0, -12], [12, 0], [0, 9], [0, -23], [0, 6], [-8, 0], [0, -14], [0, -8], [0, -14], [2, 0], [-28, 0], [55, 0], [0, 9], [0, 6], [-42, 0], [0, -18], [33, 0], [-59, 0], [0, -1], [-40, 0], [0, 0], [-19, 0], [-56, 0], [24, 0], [0, -4], [-33, 0], [0, 12], [0, -17], [0, 17], [2, 0], [24, 0], [-11, 0], [0, 20], [50, 0], [0, -10], [-52, 0], [0, -12], [0, -7], [0, -11], [-5, 0], [10, 0], [0, -11], [0, 0], [0, 22], [-37, 0], [-22, 0], [-52, 0], [0, -6], [0, -12], [-8, 0], [0, -3], [0, 2], [-4, 0], [45, 0], [0, 21], [22, 0], [0, -18], [0, -8], [20, 0], [10, 0], [0, -9], [0, 27], [0, -8], [-38, 0], [0, 0], [0, -6], [-22, 0], [0, 12], [-55, 0], [0, 13], [0, -13], [27, 0], [0, 27], [9, 0], [-35, 0], [-2, 0], [0, 16], [-33, 0], [0, 22], [-8, 0], [0, 2], [0, -22], [0, -14], [0, 11], [43, 0], [59, 0], [11, 0], [0, 21], [0, -6], [-5, 0], [0, -6], [-38, 0], [0, -1], [-30, 0], [0, -8], [0, 18], [40, 0], [0, 3], [0, -11], [-37, 0], [-35, 0], [0, -12], [14, 0], [26, 0], [0, -1], [-10, 0], [-54, 0], [0, 3], [-49, 0], [0, 6], [0, -24], [44, 0], [0, 2], [6, 0], [16, 0], [14, 0], [46, 0], [14, 0], [41, 0], [0, 13], [10, 0], [34, 0], [0, -6], [0, 2], [-52, 0], [0, 8], [2, 0], [13, 0], [0, -3], [65, 0], [0, 6], [-11, 0], [0, -9], [0, 0], [0, -26], [0, -18], [0, -8], [9, 0], [0, 5], [-41, 0], [0, -20], [0, -30], [37, 0], [64, 0], [0, -18], [4, 0], [0, 21], [24, 0], [47, 0], [0, -29], [26, 0], [0, 9], [67, 0], [0, -24], [0, -6], [0, -24], [86, 0], [0, -1], [-28, 0], [0, 22], [0, -29], [28, 0], [0, 2], [80, 0], [-73, 0], [0, 9], [-2, 0], [0, 28], [48, 0], [36, 0], [21, 0], [0, -30], [0, -8], [-73, 0], [0, -12], [51, 0], [0, 14], [74, 0], [0, 8], [-29, 0], [0, -19], [14, 0], [21, 0], [-79, 0], [0, 18], [0, 20], [78, 0], [0, -2], [0, 30], [42, 0], [34, 0], [0, 6], [0, 12], [-15, 0], [-58, 0], [0, -20], [35, 0], [17, 0], [0, -11], [0, -18], [0, -32], [0, -34], [0, -2], [62, 0], [0, 33], [0, -6], [0, 29], [52, 0], [-45, 0], [0, 9], [68, 0], [29, 0], [0, -33], [-97, 0], [0, 10], [0, -2], [73, 0], [73, 0], [0, 22], [0, 14], [80, 0], [0, -17], [82, 0], [0, -27], [0, 12], [-90, 0], [0, 10], [65, 0], [0, -18], [-60, 0], [60, 0], [-4, 0], [0, -24], [2, 0], [0, -7], [0, 37], [-8, 0], [0, 26], [-5, 0], [-39, 0], [0, 33], [86, 0], [0, 18], [-70, 0], [0, 26], [8, 0], [0, -25], [0, -3], [-49, 0], [-23, 0], [0, 18], [17, 0], [0, -12], [56, 0], [0, 30], [0, -14], [-80, 0], [-78, 0], [0, -3], [0, -9], [88, 0], [0, 20], [0, -36], [0, 16], [28, 0], [0, -3], [0, -18], [28, 0], [0, 20], [0, -30], [0, -12], [7, 0], [0, -2]]
